@@ -21,103 +21,18 @@
 #include "libfacebook.h"
 #include "fb_blist.h"
 #include "fb_connection.h"
+#include "fb_conversation.h"
 #include "fb_info.h"
 #include "fb_managefriends.h"
 #include "fb_messages.h"
 #include "fb_notifications.h"
 #include "fb_search.h"
+#include "fb_friendlist.h"
 
-/******************************************************************************/
-/* Utility functions */
-/******************************************************************************/
-
-gchar *fb_convert_unicode(const gchar *input)
-{
-	/* \u00e9t\u00e9 should be été */
-
-	gunichar unicode_char;
-	gchar unicode_char_str[6];
-	gint unicode_char_len;
-	gchar *next_pos;
-	gchar *input_string;
-	gchar *output_string;
-
-	if (input == NULL)
-		return NULL;
-
-	next_pos = input_string = g_strdup(input);
-
-	/* purple_debug_info("facebook", "unicode convert: in: %s\n", input); */
-	while ((next_pos = strstr(next_pos, "\\u")))
-	{
-		/* grab the unicode */
-		sscanf(next_pos, "\\u%4x", &unicode_char);
-		/* turn it to a char* */
-		unicode_char_len = g_unichar_to_utf8(unicode_char, unicode_char_str);
-		/* shove it back into the string */
-		g_memmove(next_pos, unicode_char_str, unicode_char_len);
-		/* move all the data after the \u0000 along */
-		g_stpcpy(next_pos + unicode_char_len, next_pos + 6);
-	}
-
-	/* purple_debug_info("facebook", "unicode convert: out: %s\n", input); */
-	output_string = g_strcompress(input_string);
-	g_free(input_string);
-
-	return output_string;
-}
-
-/* Like purple_strdup_withhtml, but escapes htmlentities too */
-gchar *fb_strdup_withhtml(const gchar *src)
-{
-	gulong destsize, i, j;
-	gchar *dest;
-
-	g_return_val_if_fail(src != NULL, NULL);
-
-	/* New length is (length of src) + (number of \n's * 3) + (number of &'s * 5) +
-		(number of <'s * 4) + (number of >'s *4) + (number of "'s * 6) -
-		(number of \r's) + 1 */
-	destsize = 1;
-	for (i = 0; src[i] != '\0'; i++)
-	{
-		if (src[i] == '\n' || src[i] == '<' || src[i] == '>')
-			destsize += 4;
-		else if (src[i] == '&')
-			destsize += 5;
-		else if (src[i] == '"')
-			destsize += 6;
-		else if (src[i] != '\r')
-			destsize++;
-	}
-
-	dest = g_malloc(destsize);
-
-	/* Copy stuff, ignoring \r's, because they are dumb */
-	for (i = 0, j = 0; src[i] != '\0'; i++) {
-		if (src[i] == '\n') {
-			strcpy(&dest[j], "<BR>");
-			j += 4;
-		} else if (src[i] == '<') {
-			strcpy(&dest[j], "&lt;");
-			j += 4;
-		} else if (src[i] == '>') {
-			strcpy(&dest[j], "&gt;");
-			j += 4;
-		} else if (src[i] == '&') {
-			strcpy(&dest[j], "&amp;");
-			j += 5;
-		} else if (src[i] == '"') {
-			strcpy(&dest[j], "&quot;");
-			j += 6;
-		} else if (src[i] != '\r')
-			dest[j++] = src[i];
-	}
-
-	dest[destsize-1] = '\0';
-
-	return dest;
-}
+static void fb_login_cb(FacebookAccount *fba, gchar *response, gsize len,
+		gpointer userdata);
+static void fb_close(PurpleConnection *pc);
+static void fb_buddy_free(PurpleBuddy *buddy);
 
 /******************************************************************************/
 /* PRPL functions */
@@ -154,12 +69,6 @@ static void fb_tooltip_text(PurpleBuddy *buddy,
 		purple_notify_user_info_add_pair(userinfo,
 				_("Status"), status);
 		g_free(status);
-		if (fbuddy->status_rel_time && *fbuddy->status_rel_time)
-		{
-			purple_notify_user_info_add_pair(userinfo,
-					_("Status changed"),
-					fbuddy->status_rel_time);
-		}
 	}
 }
 
@@ -171,13 +80,13 @@ static GList *fb_statuses(PurpleAccount *account)
 	/* Online people have a status message and also a date when it was set */
 	status = purple_status_type_new_with_attrs(PURPLE_STATUS_AVAILABLE,
 		NULL, _("Online"), FALSE, TRUE, FALSE, "message",
-		_("Message"), purple_value_new(PURPLE_TYPE_STRING),
-		"message_date", _("Message changed"),
-		purple_value_new(PURPLE_TYPE_STRING), NULL);
+		_("Message"), purple_value_new(PURPLE_TYPE_STRING), NULL);
 	types = g_list_append(types, status);
 	
 	/* Cave into feature requests and allow people to set themselves to be idle */
-	status = purple_status_type_new_full(PURPLE_STATUS_AWAY, NULL, _("Idle"), FALSE, TRUE, FALSE);
+	status = purple_status_type_new_with_attrs(PURPLE_STATUS_AWAY,
+		NULL, _("Idle"), FALSE, TRUE, FALSE, "message",
+		_("Message"), purple_value_new(PURPLE_TYPE_STRING), NULL);
 	types = g_list_append(types, status);
 
 	/* Offline people dont have messages */
@@ -200,10 +109,180 @@ static gboolean fb_get_messages_failsafe(FacebookAccount *fba)
 	return TRUE;
 }
 
+void fb_login_captcha_cancel_cb(PurpleConnection *pc, PurpleRequestFields *fields)
+{
+	purple_connection_error_reason(pc, PURPLE_CONNECTION_ERROR_OTHER_ERROR,
+		"Could not authenticate captcha.  Logging into the Facebook website may fix this.");
+}
+
+void fb_login_captcha_ok_cb(PurpleConnection *pc, PurpleRequestFields *fields)
+{
+	const gchar *captcha_response;
+	gchar *postdata, *encoded_username, *encoded_password, *encoded_charset_test,
+			*encoded_persist_data, *encoded_response, *encoded_extra_challenge,
+			*encoded_session;
+	FacebookAccount *fba = pc->proto_data;
+
+	captcha_response = purple_request_fields_get_string(fields, "captcha_response");
+	
+	encoded_response = g_strdup(purple_url_encode(captcha_response));
+	encoded_username = g_strdup(purple_url_encode(
+			purple_account_get_username(fba->account)));
+	encoded_password = g_strdup(purple_url_encode(
+			purple_account_get_password(fba->account)));
+	encoded_extra_challenge = g_strdup(purple_url_encode(
+			fba->extra_challenge));
+	encoded_persist_data = g_strdup(purple_url_encode(
+			fba->persist_data));
+	encoded_session = g_strdup(purple_url_encode(
+			fba->captcha_session));
+	encoded_charset_test = g_strdup(purple_url_encode("€,´,€,´,水,Д,Є"));
+
+	postdata = g_strdup_printf("charset_test=%s&"
+								"version=1.0&"
+								"return_session=0&"
+								"charset_test=%s&"
+								"answered_captcha=1&"
+								"captcha_persist_data=%s&"
+								"captcha_session=%s&"
+								"extra_challenge_params=%s&"
+								"captcha_response=%s&"
+								"email=%s&pass=%s&"
+								"persistent=1",
+								encoded_charset_test, encoded_charset_test,
+								encoded_persist_data, encoded_session,
+								encoded_extra_challenge, encoded_response,
+								encoded_username, encoded_password);
+	g_free(encoded_username);
+	g_free(encoded_password);
+	g_free(encoded_charset_test);
+	g_free(encoded_extra_challenge);
+	g_free(encoded_persist_data);
+	g_free(encoded_response);
+	g_free(encoded_session);
+
+	fb_post_or_get(fba, FB_METHOD_POST | FB_METHOD_SSL, "login.facebook.com",
+			"/login.php?login_attempt=1&_fb_noscript=1", postdata, fb_login_cb, NULL, FALSE);
+	g_free(postdata);
+	
+	g_free(fba->extra_challenge);
+	g_free(fba->persist_data);
+	g_free(fba->captcha_session);
+	fba->extra_challenge = NULL;
+	fba->persist_data = NULL;
+	fba->captcha_session = NULL;
+}
+
+static void fb_login_captcha_image_cb(FacebookAccount *fba, gchar *response, 
+		gsize len, gpointer userdata)
+{
+	PurpleRequestFields *fields;
+	PurpleRequestFieldGroup *group;
+	PurpleRequestField *field;
+	
+	fields = purple_request_fields_new();
+	group = purple_request_field_group_new(NULL);
+	purple_request_fields_add_group(fields, group);
+	
+	field = purple_request_field_image_new("captcha_image", "", response, len);
+	purple_request_field_group_add_field(group, field);
+	
+	field = purple_request_field_string_new("captcha_response", "", "", FALSE);
+	purple_request_field_group_add_field(group, field);
+	
+	purple_request_fields(fba->pc, 
+		_("Facebook Captcha"), _("Facebook Captcha"), 
+		_("Enter both words below, separated by a space"), 
+		fields, 
+		_("OK"), G_CALLBACK(fb_login_captcha_ok_cb), 
+		_("Logout"), G_CALLBACK(fb_login_captcha_cancel_cb), 
+		fba->account, NULL, NULL, fba->pc	 
+	);
+}
+
+static void fb_login_captcha_cb(FacebookAccount *fba, gchar *response, 
+		gsize len, gpointer userdata)
+{
+	const gchar *challenge_start = "challenge : '";
+	gchar *challenge;
+	gchar *image_url;
+	
+	challenge = g_strstr_len(response, len, challenge_start);
+	if (challenge)
+	{
+		challenge += strlen(challenge_start);
+		challenge = g_strndup(challenge, strchr(challenge, '\'') - challenge);
+		
+		image_url = g_strdup_printf("/image?c=%s", challenge);
+		
+		fb_post_or_get(fba, FB_METHOD_GET | FB_METHOD_SSL, "api-secure.recaptcha.net",
+			image_url, NULL, fb_login_captcha_image_cb, NULL, FALSE);
+	}
+}
+
 static void fb_login_cb(FacebookAccount *fba, gchar *response, gsize len,
 		gpointer userdata)
 {
 	gchar *user_cookie;
+	
+	if (len && g_strstr_len(response, len, "captcha"))
+	{
+		purple_debug_info("facebook", "captcha page: %s\n", response);
+
+		purple_connection_update_progress(fba->pc, _("Handling Captcha"), 2, 4);
+		
+		const gchar *persist_data_start = "<input type=\"hidden\" id=\"captcha_persist_data\" name=\"captcha_persist_data\" value=\"";
+		gchar *persist_data = g_strstr_len(response, len, persist_data_start);
+		if (persist_data)
+		{
+			persist_data += strlen(persist_data_start);
+			fba->persist_data = g_strndup(persist_data, strchr(persist_data, '"') - persist_data);
+		}
+		
+		const gchar *session_start = "<input type=\"hidden\" id=\"captcha_session\" name=\"captcha_session\" value=\"";
+		gchar *session = g_strstr_len(response, len, session_start);
+		if (session)
+		{
+			session += strlen(session_start);
+			fba->captcha_session = g_strndup(session, strchr(session, '"') - session);
+		}
+		
+		gchar *captcha_url;
+		const gchar *extra_challenge_params = "<input type=\"hidden\" id=\"extra_challenge_params\" name=\"extra_challenge_params\" value=\"";
+		gchar *extra_challenge = g_strstr_len(response, len, extra_challenge_params);
+		if (extra_challenge)
+		{
+			extra_challenge += strlen(extra_challenge_params);
+			fba->extra_challenge = g_strndup(extra_challenge, strchr(extra_challenge, '"') - extra_challenge);
+			extra_challenge = purple_unescape_html(fba->extra_challenge);
+			g_free(fba->extra_challenge);
+			fba->extra_challenge = extra_challenge;
+		}
+		
+		if (!fba->extra_challenge || !fba->persist_data || !fba->captcha_session)
+		{
+			purple_debug_info("facebook", "captcha response: %s\n", response);
+			g_free(fba->extra_challenge);
+			g_free(fba->persist_data);
+			g_free(fba->captcha_session);
+			fba->extra_challenge = NULL;
+			fba->persist_data = NULL;
+			fba->captcha_session = NULL;
+			purple_connection_error_reason(fba->pc, PURPLE_CONNECTION_ERROR_OTHER_ERROR,
+				"Could not authenticate captcha.  Logging into the Facebook website may fix this.");
+			return;
+		}
+		
+		captcha_url = g_strdup_printf("/challenge?k=" FACEBOOK_CAPTCHA_SITE "&%s",
+				fba->extra_challenge?fba->extra_challenge:"");
+		
+		fb_post_or_get(fba, FB_METHOD_GET | FB_METHOD_SSL, "api-secure.recaptcha.net",
+			captcha_url, NULL, fb_login_captcha_cb, NULL, FALSE);
+		
+		g_free(captcha_url);
+		
+		return;
+	}
 
 	purple_connection_update_progress(fba->pc, _("Authenticating"), 2, 3);
 
@@ -219,25 +298,20 @@ static void fb_login_cb(FacebookAccount *fba, gchar *response, gsize len,
 				_("Incorrect username or password."));
 		return;
 	}
-	fba->uid = atoi(user_cookie);
-	purple_debug_info("facebook", "uid %d\n", fba->uid);
+	fba->uid = atoll(user_cookie);
+	purple_debug_info("facebook", "uid %" G_GINT64_FORMAT "\n", fba->uid);
 
 	/* ok, we're logged in now! */
 	purple_connection_set_state(fba->pc, PURPLE_CONNECTED);
 
 	/* This will kick off our long-poll message retrieval loop */
 	fb_get_post_form_id(fba);
-	fb_get_buddy_list(fba);
 	fb_check_friend_requests(fba);
 
 	/* periodically check for people adding you to their facebook friend list */
 	fba->friend_request_timer = purple_timeout_add_seconds(60 * 5,
 			fb_check_friend_requests, fba);
 
-	/* periodically check for updates to your buddy list */
-	fba->buddy_list_timer = purple_timeout_add_seconds(60,
-			fb_get_buddy_list, fba);
-
 	/* periodically check for new notifications */
 	fba->notifications_timer = purple_timeout_add_seconds(60,
 			(GSourceFunc)fb_get_notifications_feed, fba);
@@ -255,14 +329,21 @@ static void fb_login_cb(FacebookAccount *fba, gchar *response, gsize len,
 	 */
 	fba->perpetual_messages_timer = purple_timeout_add_seconds(15,
 			(GSourceFunc)fb_get_messages_failsafe, fba);
+
+	/* init blist subsystem */
+	fb_blist_init(fba);
+
+	/* init conversation subsystem */
+	fb_conversation_init(fba);
 }
 
 static void fb_login(PurpleAccount *account)
 {
 	FacebookAccount *fba;
-	guint16 i;
-	gchar *postdata, *encoded_username, *encoded_password;
-
+	gchar *postdata, *encoded_username, *encoded_password, *encoded_charset_test;
+	const gchar* const *languages;
+	const gchar *locale;
+	
 	/* Create account and initialize state */
 	fba = g_new0(FacebookAccount, 1);
 	fba->account = account;
@@ -273,13 +354,14 @@ static void fb_login(PurpleAccount *account)
 			g_free, g_free);
 	fba->hostname_ip_cache = g_hash_table_new_full(g_str_hash, g_str_equal,
 			g_free, g_free);
+	fba->sent_messages_hash = g_hash_table_new_full(g_str_hash, g_str_equal,
+			g_free, NULL);
+	fba->auth_buddies = g_hash_table_new_full(g_str_hash, g_str_equal,
+			g_free, NULL);
 
 	g_hash_table_replace(fba->cookie_table, g_strdup("test_cookie"),
 			g_strdup("1"));
 
-	for (i = 0; i < FB_LAST_MESSAGE_MAX; i++)
-		fba->last_messages[i] = 0;
-
 	account->gc->proto_data = fba;
 
 	/* Error localized in libpurple jabber.c */
@@ -297,15 +379,23 @@ static void fb_login(PurpleAccount *account)
 			purple_account_get_username(fba->account)));
 	encoded_password = g_strdup(purple_url_encode(
 			purple_account_get_password(fba->account)));
+	encoded_charset_test = g_strdup(purple_url_encode("€,´,€,´,水,Д,Є"));
+	languages = g_get_language_names();
+	locale = languages[0];
+	if (locale == NULL || g_str_equal(locale, "C"))
+		locale = "en_US";
+
+	g_hash_table_replace(fba->cookie_table, g_strdup("lsd"), g_strdup("abcde"));
 
 	postdata = g_strdup_printf(
-			"email=%s&pass=%s&persistent=1&login=Login&charset_test=%%E2%%AC%%C2%%B4%%E2%%82%%AC%%C2%%B4%%E6%%B0%%B4%%D0%%94%%D0%%84",
-			encoded_username, encoded_password);
+			"charset_test=%s&locale=%s&email=%s&pass=%s&pass_placeHolder=Password&persistent=1&login=Login&charset_test=%s&lsd=abcde",
+			encoded_charset_test, locale, encoded_username, encoded_password, encoded_charset_test);
 	g_free(encoded_username);
 	g_free(encoded_password);
+	g_free(encoded_charset_test);
 
 	fb_post_or_get(fba, FB_METHOD_POST | FB_METHOD_SSL, "login.facebook.com",
-			"/login.php", postdata, fb_login_cb, NULL, FALSE);
+			"/login.php?login_attempt=1&_fb_noscript=1", postdata, fb_login_cb, NULL, FALSE);
 	g_free(postdata);
 }
 
@@ -313,6 +403,7 @@ static void fb_close(PurpleConnection *pc)
 {
 	FacebookAccount *fba;
 	gchar *postdata;
+	GSList *buddies;
 
 	purple_debug_info("facebook", "disconnecting account\n");
 
@@ -321,6 +412,21 @@ static void fb_close(PurpleConnection *pc)
 	
 	fba = pc->proto_data;
 
+	purple_debug_info("facebook", "unloading plugin\n");
+
+	/* destroy blist subsystem */
+	fb_blist_destroy(fba);
+
+	/* destroy conversation subsystem */
+	fb_conversation_destroy(fba);
+
+	buddies = purple_find_buddies(fba->account, NULL);
+	while(buddies) {
+		PurpleBuddy *b = buddies->data;
+		fb_buddy_free(b);
+		buddies = g_slist_delete_link(buddies, buddies);
+	}
+
 	/* Tell Facebook that we've logged out. */
 	/*
 	 * TODO
@@ -344,17 +450,16 @@ static void fb_close(PurpleConnection *pc)
 	 */
 	if (fba->post_form_id)
 		postdata = g_strdup_printf(
-				"visibility=false&post_form_id=%s",
-				fba->post_form_id);
+				"visibility=false&post_form_id=%s&"
+				"fb_dtsg=%s&post_form_id_source=AsyncRequest&"
+				"__a=1",
+				fba->post_form_id, fba->dtsg);
 	else
 		postdata = g_strdup("visibility=false");
 	fb_post_or_get(fba, FB_METHOD_POST, NULL, "/ajax/chat/settings.php",
 			postdata, NULL, NULL, FALSE);
 	g_free(postdata);
 
-	if (fba->buddy_list_timer) {
-		purple_timeout_remove(fba->buddy_list_timer);
-	}
 	if (fba->friend_request_timer) {
 		purple_timeout_remove(fba->friend_request_timer);
 	}
@@ -381,13 +486,21 @@ static void fb_close(PurpleConnection *pc)
 		fba->dns_queries = g_slist_remove(fba->dns_queries, dns_query);
 		purple_dnsquery_destroy(dns_query);
 	}
+	
+	if (fba->resending_messages != NULL) {
+		fb_cancel_resending_messages(fba);
+	}
 
 	g_hash_table_destroy(fba->cookie_table);
 	g_hash_table_destroy(fba->hostname_ip_cache);
+	g_hash_table_destroy(fba->auth_buddies);
 	g_free(fba->post_form_id);
+	g_free(fba->dtsg);
 	g_free(fba->channel_number);
-	g_slist_free(fba->auth_buddies);
 	g_free(fba->last_status_message);
+	g_free(fba->extra_challenge);
+	g_free(fba->captcha_session);
+	g_free(fba->persist_data);
 	g_free(fba);
 }
 
@@ -405,7 +518,7 @@ static unsigned int fb_send_typing(PurpleConnection *pc, const gchar *name,
 	typing_state = (state == PURPLE_TYPING) ? 1 : 0;
 
 	/* Don't send typing notifications to self */
-	if (atoi(name) != fba->uid)
+	if (atoll(name) != fba->uid)
 	{
 		encoded_name = g_strdup(purple_url_encode(name));
 		postdata = g_strdup_printf("typ=%d&to=%s&post_form_id=%s",
@@ -448,15 +561,15 @@ static void fb_set_status_ok_cb(gpointer data, const gchar *status_text)
 	if (*status_text_new != '\0')
 	{
 		status_tmp = g_strdup(purple_url_encode(status_text_new));
-		postdata = g_strdup_printf("profile_id=%d&status=%s&post_form_id=%s",
+		postdata = g_strdup_printf("profile_id=%" G_GINT64_FORMAT "&status=%s&post_form_id=%s",
 				fba->uid, status_tmp, fba->post_form_id);
 		g_free(status_tmp);
 	}
 	else
-		postdata = g_strdup_printf("profile_id=%d&clear=1&post_form_id=%s",
+		postdata = g_strdup_printf("profile_id=%" G_GINT64_FORMAT "&clear=1&post_form_id=%s",
 				fba->uid, fba->post_form_id);
 
-	fb_post_or_get(fba, FB_METHOD_POST, NULL, "/updatestatus.php",
+	fb_post_or_get(fba, FB_METHOD_POST, NULL, "/ajax/updatestatus.php",
 			postdata, NULL, NULL, FALSE);
 
 	g_free(postdata);
@@ -499,29 +612,12 @@ static void fb_buddy_free(PurpleBuddy *buddy)
 
 		g_free(fbuddy->name);
 		g_free(fbuddy->status);
-		g_free(fbuddy->status_rel_time);
 		g_free(fbuddy->thumb_url);
 		g_free(fbuddy);
 	}
 }
 
-static void fb_convo_closed(PurpleConnection *gc, const char *who)
-{
-	FacebookAccount *fba = gc->proto_data;
-	gchar *postdata;
-
-	g_return_if_fail(fba->post_form_id != NULL);
-
-	/* notify server that we closed the chat window */
-	/* close_chat=589039771&window_id=3168919846&
-	 * post_form_id=c258fe42460c7e8b61e242a37ef05afc */
-	postdata = g_strdup_printf("close_chat=%s&post_form_id=%s", who,
-			fba->post_form_id);
-	fb_post_or_get(fba, FB_METHOD_POST, NULL, "/ajax/chat/settings.php",
-			postdata, NULL, NULL, FALSE);
-	g_free(postdata);
-}
-
+#if PURPLE_MAJOR_VERSION >= 2 && PURPLE_MINOR_VERSION >= 5
 static GHashTable *fb_get_account_text_table(PurpleAccount *account)
 {
 	GHashTable *table;
@@ -532,6 +628,7 @@ static GHashTable *fb_get_account_text_table(PurpleAccount *account)
 
 	return table;
 }
+#endif
 
 /******************************************************************************/
 /* Plugin functions */
@@ -539,38 +636,11 @@ static GHashTable *fb_get_account_text_table(PurpleAccount *account)
 
 static gboolean plugin_load(PurplePlugin *plugin)
 {
-#ifdef HAVE_ZLIB
-	/* try dynamically loading zlib functions */
-	if (zlib_library == NULL)
-		/* zlib_library = dlopen("zlib1.dll", RTLD_LAZY); */
-		zlib_library = dlopen("libz.dll", RTLD_LAZY);
-	if (zlib_library == NULL)
-		zlib_library = dlopen("libz.so", RTLD_LAZY);
-	if (zlib_library == NULL)
-		zlib_library = dlopen("libz.dylib", RTLD_LAZY);
-	if (zlib_inflate == NULL && zlib_library != NULL)
-	{
-		zlib_inflate = (int (*)()) dlsym(zlib_library, "inflate");
-		zlib_inflateInit2_ = (int (*) ()) dlsym(zlib_library, "inflateInit2_");
-		zlib_inflateEnd = (int (*) ()) dlsym(zlib_library, "inflateEnd");
-	}
-#endif
-
 	return TRUE;
 }
 
 static gboolean plugin_unload(PurplePlugin *plugin)
 {
-#ifdef HAVE_ZLIB
-	if (zlib_library != NULL)
-	{
-		dlclose(zlib_library);
-		zlib_library = NULL;
-		zlib_inflate = NULL;
-		zlib_inflateInit2_ = NULL;
-		zlib_inflateEnd = NULL;
-	}
-#endif
 	return TRUE;
 }
 
@@ -580,7 +650,7 @@ static void fb_set_status_cb(PurplePluginAction *action)
 	FacebookAccount *fba = pc->proto_data;
 	gchar *uid_str;
 
-	uid_str = g_strdup_printf("%d", fba->uid);
+	uid_str = g_strdup_printf("%" G_GINT64_FORMAT, fba->uid);
 
 	purple_request_input(pc, NULL, _("Set your Facebook status"),
 			purple_account_get_alias(pc->account), "is ",
@@ -597,6 +667,17 @@ static void fb_display_plugin_info(PurplePluginAction *action)
 			_("Version"), FACEBOOK_PLUGIN_VERSION);
 }
 
+static void fb_refresh_blist(PurplePluginAction *action)
+{
+	PurpleConnection *pc;
+	FacebookAccount *fba;
+
+	pc = (PurpleConnection *) action->context;
+	fba = pc->proto_data;
+
+	fb_get_buddy_list(fba);
+}
+
 static GList *fb_actions(PurplePlugin *plugin, gpointer context)
 {
 	GList *m = NULL;
@@ -614,6 +695,11 @@ static GList *fb_actions(PurplePlugin *plugin, gpointer context)
 			fb_search_users);
 	m = g_list_append(m, act);
 
+	// TODO: remove, this is for testing.  REMOVE.
+	act = purple_plugin_action_new(_("Refresh buddy list..."),
+			fb_refresh_blist);
+	m = g_list_append(m, act);
+
 	return m;
 }
 
@@ -628,8 +714,8 @@ static GList *fb_node_menu(PurpleBlistNode *node)
 		buddy = (PurpleBuddy *)node;
 		
 		act = purple_menu_action_new(_("_Poke"),
-										PURPLE_CALLBACK(fb_blist_poke_buddy),
-										NULL, NULL);
+				PURPLE_CALLBACK(fb_blist_poke_buddy),
+				NULL, NULL);
 		m = g_list_append(m, act);
 	}
 	return m;
@@ -656,22 +742,53 @@ static void plugin_init(PurplePlugin *plugin)
 	PurplePluginProtocolInfo *prpl_info = info->extra_info;
 
 	/* Add options to the advanced screen in the account settings */
-	option = purple_account_option_bool_new(_("Hide myself in the Buddy List"), "facebook_hide_self", TRUE);
-	prpl_info->protocol_options = g_list_append(prpl_info->protocol_options, option);
+	option = purple_account_option_bool_new(
+		_("Show history in new conversations"),
+		"facebook_show_history", TRUE);
+	prpl_info->protocol_options = g_list_append(
+		prpl_info->protocol_options, option);
 
-	option = purple_account_option_bool_new(_("Set Facebook status through Pidgin status"), "facebook_set_status_through_pidgin", FALSE);
-	prpl_info->protocol_options = g_list_append(prpl_info->protocol_options, option);
+	
+	option = purple_account_option_bool_new(
+		_("Use Facebook friend-lists as Pidgin groups"),
+		"facebook_use_groups", TRUE);
+	prpl_info->protocol_options = g_list_append(
+		prpl_info->protocol_options, option);
+	
+	option = purple_account_option_bool_new(
+		_("Hide myself in the Buddy List"),
+		"facebook_hide_self", TRUE);
+	prpl_info->protocol_options = g_list_append(
+		prpl_info->protocol_options, option);
 
-	option = purple_account_option_bool_new(_("Show Facebook notifications as e-mails in Pidgin"), "facebook_get_notifications", TRUE);
-	prpl_info->protocol_options = g_list_append(prpl_info->protocol_options, option);
+	option = purple_account_option_bool_new(
+		_("Set Facebook status through Pidgin status"),
+		"facebook_set_status_through_pidgin", FALSE);
+	prpl_info->protocol_options = g_list_append(
+		prpl_info->protocol_options, option);
 
-	option = purple_account_option_bool_new(_("Edit Facebook friends from Pidgin"), "facebook_manage_friends", FALSE);
-	prpl_info->protocol_options = g_list_append(prpl_info->protocol_options, option);
+	option = purple_account_option_bool_new(
+		_("Show Facebook notifications as e-mails in Pidgin"),
+		"facebook_get_notifications", TRUE);
+	prpl_info->protocol_options = g_list_append(
+		prpl_info->protocol_options, option);
+	
+	option = purple_account_option_string_new(
+		_("Notifications RSS Feed URL"),
+		"notifications_feed_url", "");
+	prpl_info->protocol_options = g_list_append(
+		prpl_info->protocol_options, option);
+
+	option = purple_account_option_bool_new(
+		_("Edit Facebook friends from Pidgin"),
+		"facebook_manage_friends", FALSE);
+	prpl_info->protocol_options = g_list_append(
+		prpl_info->protocol_options, option);
 }
 
 static PurplePluginProtocolInfo prpl_info = {
 	/* options */
-	OPT_PROTO_UNIQUE_CHATNAME,
+	OPT_PROTO_MAIL_CHECK,
 
 	NULL,                   /* user_splits */
 	NULL,                   /* protocol_options */
@@ -696,7 +813,7 @@ static PurplePluginProtocolInfo prpl_info = {
 	NULL,                   /* change_passwd */
 	fb_add_buddy,           /* add_buddy */
 	NULL,                   /* add_buddies */
-	NULL,                   /* remove_buddy */
+	fb_buddy_remove,        /* remove_buddy */
 	NULL,                   /* remove_buddies */
 	NULL,                   /* add_permit */
 	NULL,                   /* add_deny */
@@ -715,13 +832,13 @@ static PurplePluginProtocolInfo prpl_info = {
 	NULL,                   /* get_cb_info */
 	NULL,                   /* get_cb_away */
 	NULL,                   /* alias_buddy */
-	NULL,                   /* group_buddy */
-	NULL,                   /* rename_group */
+	fb_group_buddy_move,    /* group_buddy */
+	fb_group_rename,        /* rename_group */
 	fb_buddy_free,          /* buddy_free */
-	fb_convo_closed,        /* convo_closed */
+	fb_conversation_closed, /* convo_closed */
 	purple_normalize_nocase,/* normalize */
 	NULL,                   /* set_buddy_icon */
-	NULL,                   /* remove_group */
+	fb_group_remove,        /* remove_group */
 	NULL,                   /* get_cb_real_name */
 	NULL,                   /* set_chat_topic */
 	NULL,                   /* find_blist_chat */
@@ -738,35 +855,39 @@ static PurplePluginProtocolInfo prpl_info = {
 	NULL,                   /* unregister_user */
 	NULL,                   /* send_attention */
 	NULL,                   /* attention_types */
+#if PURPLE_MAJOR_VERSION >= 2 && PURPLE_MINOR_VERSION >= 5
 	sizeof(PurplePluginProtocolInfo), /* struct_size */
 	fb_get_account_text_table, /* get_account_text_table */
+#else
+	(gpointer) sizeof(PurplePluginProtocolInfo)
+#endif
 };
 
 static PurplePluginInfo info = {
 	PURPLE_PLUGIN_MAGIC,
-	2, /* major_version */
-	3, /* minor version */
-	PURPLE_PLUGIN_PROTOCOL, /* type */
-	NULL, /* ui_requirement */
-	0, /* flags */
-	NULL, /* dependencies */
-	PURPLE_PRIORITY_DEFAULT, /* priority */
-	"prpl-bigbrownchunx-facebookim", /* id */
-	"Facebook", /* name */
-	FACEBOOK_PLUGIN_VERSION, /* version */
-	N_("Facebook Protocol Plugin"), /* summary */
-	N_("Facebook Protocol Plugin"), /* description */
-	"Eion Robb <eionrobb@gmail.com>", /* author */
-	"http://pidgin-facebookchat.googlecode.com/", /* homepage */
-	plugin_load, /* load */
-	plugin_unload, /* unload */
-	NULL, /* destroy */
-	NULL, /* ui_info */
-	&prpl_info, /* extra_info */
-	NULL, /* prefs_info */
-	fb_actions, /* actions */
+	2,						/* major_version */
+	3, 						/* minor version */
+	PURPLE_PLUGIN_PROTOCOL, 			/* type */
+	NULL, 						/* ui_requirement */
+	0, 						/* flags */
+	NULL, 						/* dependencies */
+	PURPLE_PRIORITY_DEFAULT, 			/* priority */
+	FACEBOOK_PLUGIN_ID,				/* id */
+	"Facebook", 					/* name */
+	FACEBOOK_PLUGIN_VERSION, 			/* version */
+	N_("Facebook Protocol Plugin"), 		/* summary */
+	N_("Facebook Protocol Plugin"), 		/* description */
+	"Eion Robb <eionrobb@gmail.com>", 		/* author */
+	"http://pidgin-facebookchat.googlecode.com/",	/* homepage */
+	plugin_load, 					/* load */
+	plugin_unload, 					/* unload */
+	NULL, 						/* destroy */
+	NULL, 						/* ui_info */
+	&prpl_info, 					/* extra_info */
+	NULL, 						/* prefs_info */
+	fb_actions, 					/* actions */
 
-	/* padding */
+							/* padding */
 	NULL,
 	NULL,
 	NULL,
